@@ -89,7 +89,6 @@ static void                  insert_hf                               (gpointer k
                                                                       gpointer value,
                                                                       gpointer data);
 static AtkKeyEventStruct *  atk_key_event_from_clutter_event_key     (ClutterKeyEvent *event);
-static void                 do_window_event_initialization           (void);
 
 
 /* This is just a copy of the Gail one, a shared library or place to
@@ -165,18 +164,13 @@ cally_util_get_root (void)
 static const gchar *
 cally_util_get_toolkit_name (void)
 {
-  return "CALLY";
+  return "clutter";
 }
 
 static const gchar *
 cally_util_get_toolkit_version (void)
 {
-  /*
-   * FIXME:
-   * Version is passed in as a -D flag when this file is
-   * compiled.
-   */
-  return "0.1";
+  return CLUTTER_VERSION_S;
 }
 
 
@@ -189,33 +183,10 @@ cally_util_add_global_event_listener (GSignalEmissionHook listener,
 
   split_string = g_strsplit (event_type, ":", 3);
 
-  if (split_string)
-    {
-      if (!strcmp ("window", split_string[0]))
-        {
-          /* Using ClutterStage as the window equivalent, although
-             several methods (move, etc) are missing. This would be
-             probably defined for other window-related classes (MxWindow)
+  if (g_strv_length (split_string) == 3)
+    rc = add_listener (listener, split_string[1], split_string[2], event_type);
 
-             FIXME: for this reason, this process should be extendable
-             on the future.*/
-          static gboolean initialized = FALSE;
-
-          if (initialized == FALSE)
-            {
-              do_window_event_initialization ();
-              initialized = TRUE;
-            }
-
-          rc = add_listener (listener, "CallyStage", split_string[1], event_type);
-        }
-      else
-        {
-          rc = add_listener (listener, split_string[1], split_string[2], event_type);
-        }
-
-      g_strfreev (split_string);
-    }
+  g_strfreev (split_string);
 
   return rc;
 }
@@ -548,12 +519,3 @@ cally_util_stage_removed_cb (ClutterStageManager *stage_manager,
 
   g_signal_handlers_disconnect_by_func (stage, cally_key_snooper_cb, NULL);
 }
-
-static void
-do_window_event_initialization (void)
-{
-  /*
-   * Ensure that CallyStageClass exists.
-   */
-  g_type_class_unref (g_type_class_ref (CALLY_TYPE_STAGE));
-}
