@@ -95,8 +95,6 @@ static gboolean clutter_clock_dispatch (GSource     *source,
                                         GSourceFunc  callback,
                                         gpointer     user_data);
 
-static ClutterMasterClock *default_clock = NULL;
-
 static GSourceFuncs clock_funcs = {
   clutter_clock_prepare,
   clutter_clock_check,
@@ -277,6 +275,20 @@ clutter_clock_prepare (GSource *source,
   int delay;
 
   clutter_threads_enter ();
+
+  if (G_UNLIKELY (clutter_paint_debug_flags &
+                  CLUTTER_DEBUG_CONTINUOUS_REDRAW))
+    {
+      ClutterStageManager *stage_manager = clutter_stage_manager_get_default ();
+      const GSList *stages, *l;
+
+      stages = clutter_stage_manager_peek_stages (stage_manager);
+
+      /* Queue a full redraw on all of the stages */
+      for (l = stages; l != NULL; l = l->next)
+        clutter_actor_queue_redraw (l->data);
+    }
+
   delay = master_clock_next_frame_delay (master_clock);
   clutter_threads_leave ();
 
@@ -447,12 +459,12 @@ clutter_master_clock_init (ClutterMasterClock *self)
 ClutterMasterClock *
 _clutter_master_clock_get_default (void)
 {
-  if (G_LIKELY (default_clock != NULL))
-    return default_clock;
+  ClutterMainContext *context = _clutter_context_get_default ();
 
-  default_clock = g_object_new (CLUTTER_TYPE_MASTER_CLOCK, NULL);
+  if (G_UNLIKELY (context->master_clock == NULL))
+    context->master_clock = g_object_new (CLUTTER_TYPE_MASTER_CLOCK, NULL);
 
-  return default_clock;
+  return context->master_clock;
 }
 
 /*
@@ -518,8 +530,8 @@ _clutter_master_clock_start_running (ClutterMasterClock *master_clock)
  * @master_clock: a #ClutterMasterClock
  *
  * Advances all the timelines held by the master clock. This function
- * should be called before calling clutter_redraw() to make sure that
- * all the timelines are advanced and the scene is updated.
+ * should be called before calling _clutter_stage_do_update() to
+ * make sure that all the timelines are advanced and the scene is updated.
  */
 void
 _clutter_master_clock_advance (ClutterMasterClock *master_clock)
