@@ -30,35 +30,37 @@
 #include "config.h"
 #endif
 
-#include "clutter-stage-cogl.h"
-#include "clutter-backend-cogl.h"
+#include "clutter-config.h"
 
+#include "clutter-stage-cogl.h"
+
+#include "clutter-actor-private.h"
+#include "clutter-backend-private.h"
 #include "clutter-debug.h"
 #include "clutter-event.h"
 #include "clutter-enum-types.h"
 #include "clutter-feature.h"
 #include "clutter-main.h"
 #include "clutter-private.h"
-#include "clutter-actor-private.h"
+#include "clutter-profile.h"
 #include "clutter-stage-private.h"
 #include "clutter-util.h"
 
-#ifdef COGL_HAS_X11_SUPPORT
-static ClutterStageWindowIface *clutter_stage_window_parent_iface = NULL;
-#endif
-
 static void clutter_stage_window_iface_init (ClutterStageWindowIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (ClutterStageCogl,
                          _clutter_stage_cogl,
-#ifdef COGL_HAS_X11_SUPPORT
-                         CLUTTER_TYPE_STAGE_X11,
-#else
                          G_TYPE_OBJECT,
-#endif
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_STAGE_WINDOW,
                                                 clutter_stage_window_iface_init));
 
+enum {
+  PROP_0,
+  PROP_WRAPPER,
+  PROP_BACKEND,
+  PROP_LAST
+};
+
 static void
 clutter_stage_cogl_unrealize (ClutterStageWindow *stage_window)
 {
@@ -66,11 +68,6 @@ clutter_stage_cogl_unrealize (ClutterStageWindow *stage_window)
 
   CLUTTER_NOTE (BACKEND, "Unrealizing Cogl stage [%p]", stage_cogl);
 
-#ifdef COGL_HAS_XLIB_SUPPORT
-  /* chain up to the StageX11 implementation */
-  clutter_stage_window_parent_iface->unrealize (stage_window);
-#endif
-
   if (stage_cogl->onscreen != NULL)
     {
       cogl_object_unref (stage_cogl->onscreen);
@@ -101,15 +98,11 @@ static gboolean
 clutter_stage_cogl_realize (ClutterStageWindow *stage_window)
 {
   ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
-#ifdef COGL_HAS_XLIB_SUPPORT
-  ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (stage_window);
-#endif
   ClutterBackend *backend;
   CoglFramebuffer *framebuffer;
   GError *error = NULL;
   gfloat width = 800;
   gfloat height = 600;
-  const char *clutter_vblank;
 
   CLUTTER_NOTE (BACKEND, "Realizing stage '%s' [%p]",
                 G_OBJECT_TYPE_NAME (stage_cogl),
@@ -117,27 +110,14 @@ clutter_stage_cogl_realize (ClutterStageWindow *stage_window)
 
   backend = clutter_get_default_backend ();
 
-#ifdef COGL_HAS_XLIB_SUPPORT
-  clutter_actor_get_size (CLUTTER_ACTOR (stage_x11->wrapper), &width, &height);
-#endif
-
-  stage_cogl->onscreen = cogl_onscreen_new (backend->cogl_context,
-                                            width, height);
-
-#ifdef COGL_HAS_XLIB_SUPPORT
-  if (stage_x11->xwin != None)
+  if (stage_cogl->onscreen == NULL)
     {
-      cogl_x11_onscreen_set_foreign_window_xid (stage_cogl->onscreen,
-                                                stage_x11->xwin,
-                                                _clutter_stage_x11_update_foreign_event_mask,
-                                                stage_x11);
-
+      stage_cogl->onscreen = cogl_onscreen_new (backend->cogl_context,
+						width, height);
     }
-#endif
 
-  clutter_vblank = _clutter_backend_cogl_get_vblank ();
-  if (clutter_vblank && strcmp (clutter_vblank, "none") == 0)
-    cogl_onscreen_set_swap_throttled (stage_cogl->onscreen, FALSE);
+  cogl_onscreen_set_swap_throttled (stage_cogl->onscreen,
+                                    _clutter_get_sync_to_vblank ());
 
   framebuffer = COGL_FRAMEBUFFER (stage_cogl->onscreen);
   if (!cogl_framebuffer_allocate (framebuffer, &error))
@@ -149,7 +129,7 @@ clutter_stage_cogl_realize (ClutterStageWindow *stage_window)
       return FALSE;
     }
 
-  /* FIXME: for fullscreen Cogl platforms then the size we gave above
+  /* FIXME: for fullscreen Cogl platforms then the size we gave
    * will be ignored, so we need to make sure the stage size is
    * updated to this size. */
 
@@ -161,14 +141,7 @@ clutter_stage_cogl_realize (ClutterStageWindow *stage_window)
                                                     stage_cogl);
     }
 
-#ifdef COGL_HAS_XLIB_SUPPORT
-  if (stage_x11->xwin == None)
-    stage_x11->xwin = cogl_x11_onscreen_get_window_xid (stage_cogl->onscreen);
-
-  return clutter_stage_window_parent_iface->realize (stage_window);
-#else
   return TRUE;
-#endif
 }
 
 static int
@@ -179,8 +152,6 @@ clutter_stage_cogl_get_pending_swaps (ClutterStageWindow *stage_window)
   return stage_cogl->pending_swaps;
 }
 
-#ifndef COGL_HAS_XLIB_SUPPORT
-
 static ClutterActor *
 clutter_stage_cogl_get_wrapper (ClutterStageWindow *stage_window)
 {
@@ -189,7 +160,7 @@ clutter_stage_cogl_get_wrapper (ClutterStageWindow *stage_window)
 
 static void
 clutter_stage_cogl_show (ClutterStageWindow *stage_window,
-                        gboolean            do_raise)
+			 gboolean            do_raise)
 {
   ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
 
@@ -205,8 +176,8 @@ clutter_stage_cogl_hide (ClutterStageWindow *stage_window)
 }
 
 static void
-clutter_stage_cogl_get_geometry (ClutterStageWindow *stage_window,
-                                ClutterGeometry    *geometry)
+clutter_stage_cogl_get_geometry (ClutterStageWindow    *stage_window,
+                                 cairo_rectangle_int_t *geometry)
 {
   ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
 
@@ -238,8 +209,6 @@ clutter_stage_cogl_resize (ClutterStageWindow *stage_window,
 {
 }
 
-#endif /* COGL_HAS_XLIB_SUPPORT */
-
 static gboolean
 clutter_stage_cogl_has_redraw_clips (ClutterStageWindow *stage_window)
 {
@@ -288,8 +257,8 @@ clutter_stage_cogl_ignoring_redraw_clips (ClutterStageWindow *stage_window)
  *   buffer.
  */
 static void
-clutter_stage_cogl_add_redraw_clip (ClutterStageWindow *stage_window,
-                                   ClutterGeometry    *stage_clip)
+clutter_stage_cogl_add_redraw_clip (ClutterStageWindow    *stage_window,
+                                    cairo_rectangle_int_t *stage_clip)
 {
   ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
 
@@ -313,32 +282,27 @@ clutter_stage_cogl_add_redraw_clip (ClutterStageWindow *stage_window,
 
   if (!stage_cogl->initialized_redraw_clip)
     {
-      stage_cogl->bounding_redraw_clip.x = stage_clip->x;
-      stage_cogl->bounding_redraw_clip.y = stage_clip->y;
-      stage_cogl->bounding_redraw_clip.width = stage_clip->width;
-      stage_cogl->bounding_redraw_clip.height = stage_clip->height;
+      stage_cogl->bounding_redraw_clip = *stage_clip;
     }
   else if (stage_cogl->bounding_redraw_clip.width > 0)
     {
-      clutter_geometry_union (&stage_cogl->bounding_redraw_clip, stage_clip,
-			      &stage_cogl->bounding_redraw_clip);
+      _clutter_util_rectangle_union (&stage_cogl->bounding_redraw_clip,
+                                     stage_clip,
+                                     &stage_cogl->bounding_redraw_clip);
     }
 
   stage_cogl->initialized_redraw_clip = TRUE;
 }
 
 static gboolean
-clutter_stage_cogl_get_redraw_clip_bounds (ClutterStageWindow *stage_window,
+clutter_stage_cogl_get_redraw_clip_bounds (ClutterStageWindow    *stage_window,
                                            cairo_rectangle_int_t *stage_clip)
 {
   ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
 
   if (stage_cogl->using_clipped_redraw)
     {
-      stage_clip->x = stage_cogl->bounding_redraw_clip.x;
-      stage_clip->y = stage_cogl->bounding_redraw_clip.y;
-      stage_clip->width = stage_cogl->bounding_redraw_clip.width;
-      stage_clip->height = stage_cogl->bounding_redraw_clip.height;
+      *stage_clip = stage_cogl->bounding_redraw_clip;
 
       return TRUE;
     }
@@ -351,11 +315,10 @@ static void
 clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
 {
   ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
-  ClutterActor *wrapper;
-  ClutterBackend *backend;
-  ClutterBackendCogl *backend_cogl;
   gboolean may_use_clipped_redraw;
   gboolean use_clipped_redraw;
+  gboolean can_blit_sub_buffer;
+  ClutterActor *wrapper;
 
   CLUTTER_STATIC_TIMER (painting_timer,
                         "Redrawing", /* parent */
@@ -373,40 +336,27 @@ clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
                         "The time spent in blit_sub_buffer",
                         0 /* no application private data */);
 
-#ifdef COGL_HAS_X11_SUPPORT
-  ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (stage_cogl);
-
-  wrapper = CLUTTER_ACTOR (stage_x11->wrapper);
-#else
   wrapper = CLUTTER_ACTOR (stage_cogl->wrapper);
-#endif
 
   if (!stage_cogl->onscreen)
     return;
 
-  backend = clutter_get_default_backend ();
-  backend_cogl = CLUTTER_BACKEND_COGL (backend);
-
   CLUTTER_TIMER_START (_clutter_uprof_context, painting_timer);
 
-  if (G_LIKELY (backend_cogl->can_blit_sub_buffer) &&
+  can_blit_sub_buffer =
+    cogl_clutter_winsys_has_feature (COGL_WINSYS_FEATURE_SWAP_REGION);
+
+  may_use_clipped_redraw = FALSE;
+  if (_clutter_stage_window_can_clip_redraws (stage_window) &&
+      can_blit_sub_buffer &&
       /* NB: a zero width redraw clip == full stage redraw */
       stage_cogl->bounding_redraw_clip.width != 0 &&
       /* some drivers struggle to get going and produce some junk
        * frames when starting up... */
-      G_LIKELY (stage_cogl->frame_count > 3)
-#ifdef COGL_HAS_X11_SUPPORT
-      /* While resizing a window clipped redraws are disabled to avoid
-       * artefacts. See clutter-event-x11.c:event_translate for a
-       * detailed explanation */
-      && G_LIKELY (stage_x11->clipped_redraws_cool_off == 0)
-#endif
-      )
+      stage_cogl->frame_count > 3)
     {
       may_use_clipped_redraw = TRUE;
     }
-  else
-    may_use_clipped_redraw = FALSE;
 
   if (may_use_clipped_redraw &&
       G_LIKELY (!(clutter_paint_debug_flags &
@@ -457,7 +407,7 @@ clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
       G_UNLIKELY ((clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS)))
     {
       static CoglMaterial *outline = NULL;
-      ClutterGeometry *clip = &stage_cogl->bounding_redraw_clip;
+      cairo_rectangle_int_t *clip = &stage_cogl->bounding_redraw_clip;
       ClutterActor *actor = CLUTTER_ACTOR (wrapper);
       CoglHandle vbo;
       float x_1 = clip->x;
@@ -504,7 +454,7 @@ clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
   /* push on the screen */
   if (use_clipped_redraw)
     {
-      ClutterGeometry *clip = &stage_cogl->bounding_redraw_clip;
+      cairo_rectangle_int_t *clip = &stage_cogl->bounding_redraw_clip;
       int copy_area[4];
 
       /* XXX: It seems there will be a race here in that the stage
@@ -569,16 +519,6 @@ clutter_stage_cogl_get_active_framebuffer (ClutterStageWindow *stage_window)
 static void
 clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
 {
-#ifdef COGL_HAS_X11_SUPPORT
-  clutter_stage_window_parent_iface = g_type_interface_peek_parent (iface);
-
-  iface->realize = clutter_stage_cogl_realize;
-  iface->unrealize = clutter_stage_cogl_unrealize;
-
-  /* the rest is inherited from ClutterStageX11 */
-
-#else /* COGL_HAS_X11_SUPPORT */
-
   iface->realize = clutter_stage_cogl_realize;
   iface->unrealize = clutter_stage_cogl_unrealize;
   iface->get_wrapper = clutter_stage_cogl_get_wrapper;
@@ -586,9 +526,6 @@ clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
   iface->resize = clutter_stage_cogl_resize;
   iface->show = clutter_stage_cogl_show;
   iface->hide = clutter_stage_cogl_hide;
-
-#endif /* COGL_HAS_X11_SUPPORT */
-
   iface->get_pending_swaps = clutter_stage_cogl_get_pending_swaps;
   iface->add_redraw_clip = clutter_stage_cogl_add_redraw_clip;
   iface->has_redraw_clips = clutter_stage_cogl_has_redraw_clips;
@@ -598,11 +535,28 @@ clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
   iface->get_active_framebuffer = clutter_stage_cogl_get_active_framebuffer;
 }
 
-#ifdef COGL_HAS_X11_SUPPORT
 static void
-clutter_stage_cogl_dispose (GObject *gobject)
+clutter_stage_cogl_set_property (GObject      *gobject,
+				 guint         prop_id,
+				 const GValue *value,
+				 GParamSpec   *pspec)
 {
-  G_OBJECT_CLASS (_clutter_stage_cogl_parent_class)->dispose (gobject);
+  ClutterStageCogl *self = CLUTTER_STAGE_COGL (gobject);
+
+  switch (prop_id)
+    {
+    case PROP_WRAPPER:
+      self->wrapper = g_value_get_object (value);
+      break;
+
+    case PROP_BACKEND:
+      self->backend = g_value_get_object (value);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
+      break;
+    }
 }
 
 static void
@@ -610,14 +564,11 @@ _clutter_stage_cogl_class_init (ClutterStageCoglClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  gobject_class->dispose = clutter_stage_cogl_dispose;
+  gobject_class->set_property = clutter_stage_cogl_set_property;
+
+  g_object_class_override_property (gobject_class, PROP_WRAPPER, "wrapper");
+  g_object_class_override_property (gobject_class, PROP_BACKEND, "backend");
 }
-#else
-static void
-_clutter_stage_cogl_class_init (ClutterStageCoglClass *klass)
-{
-}
-#endif /* COGL_HAS_X11_SUPPORT */
 
 static void
 _clutter_stage_cogl_init (ClutterStageCogl *stage)
