@@ -44,6 +44,7 @@
 #include "clutter-input-device-evdev.h"
 #include "clutter-main.h"
 #include "clutter-private.h"
+#include "clutter-stage-manager.h"
 #include "clutter-xkb-utils.h"
 
 #include "clutter-device-manager-evdev.h"
@@ -268,9 +269,14 @@ clutter_event_dispatch (GSource     *g_source,
   ClutterEvent *event;
   gint len, i, dx = 0, dy = 0;
   uint32_t _time;
+  ClutterStageManager *stage_manager;
+  ClutterStage *default_stage;
 
   clutter_threads_enter ();
 
+  stage_manager = clutter_stage_manager_get_default ();
+  default_stage = clutter_stage_manager_get_default_stage (stage_manager);
+
   /* Don't queue more events if we haven't finished handling the previous batch
    */
   if (!clutter_events_pending ())
@@ -303,6 +309,10 @@ clutter_event_dispatch (GSource     *g_source,
          goto out;
        }
 
+       /* Drop events if we don't have any stage to forward them to */
+       if (!default_stage)
+         goto out;
+
        for (i = 0; i < len / sizeof (ev[0]); i++)
          {
            struct input_event *e = &ev[i];
@@ -530,7 +540,6 @@ evdev_add_device (ClutterDeviceManagerEvdev *manager_evdev,
   ClutterDeviceManager *manager = (ClutterDeviceManager *) manager_evdev;
   ClutterInputDeviceType type = CLUTTER_EXTENSION_DEVICE;
   ClutterInputDevice *device;
-  ClutterActor *stage;
   const gchar *device_file, *sysfs_path, *device_name;
 
   device_file = g_udev_device_get_device_file (udev_device);
@@ -573,10 +582,6 @@ evdev_add_device (ClutterDeviceManagerEvdev *manager_evdev,
                          "enabled", TRUE,
                          NULL);
 
-  /* Always associate the device to the default stage */
-  stage = clutter_stage_get_default ();
-  _clutter_input_device_set_stage (device, CLUTTER_STAGE (stage));
-
   _clutter_device_manager_add_device (manager, device);
 
   CLUTTER_NOTE (EVENT, "Added device %s, type %d, sysfs %s",
@@ -847,25 +852,18 @@ clutter_device_manager_evdev_init (ClutterDeviceManagerEvdev *self)
   self->priv = CLUTTER_DEVICE_MANAGER_EVDEV_GET_PRIVATE (self);
 }
 
-/*
- * _clutter_events_evdev_init() and _clutter_events_evdev_uninit() are the two
- * symbol to use the evdev event backend from the EGL backend
- */
-
 void
 _clutter_events_evdev_init (ClutterBackend *backend)
 {
   CLUTTER_NOTE (EVENT, "Initializing evdev backend");
 
-  /* We just have to create the singleon here */
-  clutter_device_manager_get_default ();
+  backend->device_manager = g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_EVDEV,
+                                          "backend", backend,
+                                          NULL);
 }
 
 void
 _clutter_events_evdev_uninit (ClutterBackend *backend)
 {
-  ClutterDeviceManager *manager;
-
-  manager = clutter_device_manager_get_default ();
-  g_object_unref (manager);
+  CLUTTER_NOTE (EVENT, "Uninitializing evdev backend");
 }
