@@ -28,6 +28,7 @@
 #include "clutter-device-manager-osx.h"
 #include "clutter-shader.h"
 #include "clutter-stage-osx.h"
+#include "clutter-event-loop-osx.h"
 
 #include "clutter-debug.h"
 #include "clutter-private.h"
@@ -37,6 +38,8 @@
 
 #import <AppKit/AppKit.h>
 
+#define clutter_backend_osx_get_type    _clutter_backend_osx_get_type
+
 G_DEFINE_TYPE (ClutterBackendOSX, clutter_backend_osx, CLUTTER_TYPE_BACKEND)
 
 /*************************************************************************/
@@ -68,52 +71,12 @@ clutter_backend_osx_post_parse (ClutterBackend  *backend,
 static ClutterFeatureFlags
 clutter_backend_osx_get_features (ClutterBackend *backend)
 {
-  return CLUTTER_FEATURE_STAGE_MULTIPLE|CLUTTER_FEATURE_STAGE_USER_RESIZE;
+  return CLUTTER_FEATURE_STAGE_MULTIPLE
+       | CLUTTER_FEATURE_STAGE_USER_RESIZE;
 }
 
-static ClutterStageWindow*
-clutter_backend_osx_create_stage (ClutterBackend  *backend,
-                                  ClutterStage    *wrapper,
-                                  GError         **error)
-{
-  ClutterStageWindow *impl;
-
-  CLUTTER_OSX_POOL_ALLOC();
-
-  impl = _clutter_stage_osx_new (backend, wrapper);
-
-  CLUTTER_NOTE (BACKEND, "create_stage: wrapper=%p - impl=%p",
-                wrapper,
-                impl);
-
-  CLUTTER_OSX_POOL_RELEASE();
-
-  return impl;
-}
-
-static inline void
-clutter_backend_osx_create_device_manager (ClutterBackendOSX *backend_osx)
-{
-  if (backend_osx->device_manager != NULL)
-    return;
-
-  backend_osx->device_manager = g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_OSX,
-                                              "backend", CLUTTER_BACKEND(backend_osx),
-                                              NULL);
-}
-
-static ClutterDeviceManager *
-clutter_backend_osx_get_device_manager (ClutterBackend *backend)
-{
-  ClutterBackendOSX *backend_osx = CLUTTER_BACKEND_OSX (backend);
-
-  clutter_backend_osx_create_device_manager (backend_osx);
-
-  return backend_osx->device_manager;
-}
-
-static void
-clutter_backend_osx_init_events (ClutterBackend *backend)
+void
+_clutter_backend_osx_events_init (ClutterBackend *backend)
 {
   ClutterBackendOSX *backend_osx = CLUTTER_BACKEND_OSX (backend);
 
@@ -122,8 +85,12 @@ clutter_backend_osx_init_events (ClutterBackend *backend)
 
   CLUTTER_NOTE (BACKEND, "init_events");
 
-  clutter_backend_osx_create_device_manager (backend_osx);
-  _clutter_events_osx_init ();
+  backend->device_manager = backend_osx->device_manager =
+    g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_OSX,
+                  "backend", CLUTTER_BACKEND(backend_osx),
+                  NULL);
+
+  _clutter_osx_event_loop_init ();
 }
 
 static gboolean
@@ -141,7 +108,7 @@ clutter_backend_osx_create_context (ClutterBackend  *backend,
        */
       NSOpenGLPixelFormatAttribute attrs[] = {
         NSOpenGLPFADoubleBuffer,
-        NSOpenGLPFADepthSize, 24,
+        NSOpenGLPFADepthSize, 32,
         NSOpenGLPFAStencilSize, 8,
         0
       };
@@ -188,6 +155,7 @@ clutter_backend_osx_ensure_context (ClutterBackend *backend,
       g_assert (CLUTTER_IS_STAGE_OSX (impl));
       stage_osx = CLUTTER_STAGE_OSX (impl);
 
+      [backend_osx->context clearDrawable];
       [backend_osx->context setView:stage_osx->view];
       [backend_osx->context makeCurrentContext];
     }
@@ -228,8 +196,6 @@ clutter_backend_osx_dispose (GObject *object)
 {
   ClutterBackendOSX *self = CLUTTER_BACKEND_OSX (object);
 
-  _clutter_shader_release_all ();
-
   [self->context release];
   self->context = NULL;
 
@@ -247,17 +213,10 @@ clutter_backend_osx_class_init (ClutterBackendOSXClass *klass)
 
   object_class->dispose = clutter_backend_osx_dispose;
 
+  backend_class->stage_window_type = CLUTTER_TYPE_STAGE_OSX;
+
   backend_class->post_parse         = clutter_backend_osx_post_parse;
   backend_class->get_features       = clutter_backend_osx_get_features;
-  backend_class->create_stage       = clutter_backend_osx_create_stage;
   backend_class->create_context     = clutter_backend_osx_create_context;
   backend_class->ensure_context     = clutter_backend_osx_ensure_context;
-  backend_class->init_events        = clutter_backend_osx_init_events;
-  backend_class->get_device_manager = clutter_backend_osx_get_device_manager;
-}
-
-GType
-_clutter_backend_impl_get_type (void)
-{
-  return clutter_backend_osx_get_type ();
 }
