@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2007  Tommi Komulainen <tommi.komulainen@iki.fi>
  * Copyright (C) 2007  OpenedHand Ltd.
+ * Copyright (C) 2011  Crystalnix <vgachkaylo@crystalnix.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,12 +24,13 @@
 #define __CLUTTER_BACKEND_OSX_H__
 
 #include "clutter-backend-private.h"
+#include "clutter-device-manager.h"
 
 @class NSOpenGLPixelFormat, NSOpenGLContext;
 
 G_BEGIN_DECLS
 
-#define CLUTTER_TYPE_BACKEND_OSX             (clutter_backend_osx_get_type())
+#define CLUTTER_TYPE_BACKEND_OSX             (_clutter_backend_osx_get_type())
 #define CLUTTER_BACKEND_OSX(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj),CLUTTER_TYPE_BACKEND_OSX,ClutterBackendOSX))
 #define CLUTTER_BACKEND_OSX_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass),CLUTTER_TYPE_BACKEND_OSX,ClutterBackend))
 #define CLUTTER_IS_BACKEND_OSX(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj),CLUTTER_TYPE_BACKEND_OSX))
@@ -42,8 +44,9 @@ struct _ClutterBackendOSX
 {
   ClutterBackend parent;
 
-  NSOpenGLPixelFormat *pixel_format;
-  NSOpenGLContext     *context;
+  NSOpenGLPixelFormat  *pixel_format;
+  NSOpenGLContext      *context;
+  ClutterDeviceManager *device_manager;
 };
 
 struct _ClutterBackendOSXClass
@@ -51,7 +54,9 @@ struct _ClutterBackendOSXClass
   ClutterBackendClass parent_class;
 };
 
-GType        clutter_backend_osx_get_type    (void) G_GNUC_CONST;
+GType _clutter_backend_osx_get_type (void) G_GNUC_CONST;
+
+void            _clutter_backend_osx_events_init        (ClutterBackend *backend);
 
 G_END_DECLS
 
