@@ -32,6 +32,14 @@
 
 #import <AppKit/AppKit.h>
 
+enum
+{
+  PROP_0,
+
+  PROP_BACKEND,
+  PROP_WRAPPER
+};
+
 static void clutter_stage_window_iface_init (ClutterStageWindowIface *iface);
 
 #define clutter_stage_osx_get_type      _clutter_stage_osx_get_type
@@ -120,7 +128,7 @@ clutter_stage_osx_get_wrapper (ClutterStageWindow *stage_window);
 
 - (NSSize) windowWillResize:(NSWindow *) sender toSize:(NSSize) frameSize
 {
-  if ( clutter_stage_get_user_resizable (self->stage_osx->wrapper) )
+  if (clutter_stage_get_user_resizable (self->stage_osx->wrapper))
     {
       guint min_width, min_height;
       clutter_stage_get_minimum_size (self->stage_osx->wrapper,
@@ -135,7 +143,7 @@ clutter_stage_osx_get_wrapper (ClutterStageWindow *stage_window);
 
 - (void)windowDidChangeScreen:(NSNotification *)notification
 {
-  clutter_redraw(CLUTTER_STAGE(self->stage_osx->wrapper));
+  clutter_stage_ensure_redraw (CLUTTER_STAGE (self->stage_osx->wrapper));
 }
 @end
 
@@ -195,8 +203,8 @@ clutter_stage_osx_get_wrapper (ClutterStageWindow *stage_window);
   stage_osx->requisition_width = [self bounds].size.width;
   stage_osx->requisition_height = [self bounds].size.height;
   clutter_actor_set_size (CLUTTER_ACTOR (self->stage_osx->wrapper),
-                          (int)[self bounds].size.width,
-                          (int)[self bounds].size.height);
+                          stage_osx->requisition_width,
+                          stage_osx->requisition_height);
 
   [self removeTrackingRect:tracking_rect];
   tracking_rect = [self addTrackingRect:[self bounds]
@@ -434,8 +442,8 @@ clutter_stage_osx_hide (ClutterStageWindow *stage_window)
 }
 
 static void
-clutter_stage_osx_get_geometry (ClutterStageWindow *stage_window,
-                                ClutterGeometry    *geometry)
+clutter_stage_osx_get_geometry (ClutterStageWindow    *stage_window,
+                                cairo_rectangle_int_t *geometry)
 {
   ClutterBackend *backend = clutter_get_default_backend ();
   ClutterStageOSX *self = CLUTTER_STAGE_OSX (stage_window);
@@ -601,24 +609,6 @@ clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
   iface->redraw             = clutter_stage_osx_redraw;
 }
 
-/*************************************************************************/
-ClutterStageWindow *
-_clutter_stage_osx_new (ClutterBackend *backend,
-                        ClutterStage   *wrapper)
-{
-  ClutterStageOSX *self;
-
-  self = g_object_new (CLUTTER_TYPE_STAGE_OSX, NULL);
-  self->backend = backend;
-  self->wrapper = wrapper;
-  self->isHiding = false;
-  self->haveRealized = false;
-  self->view = NULL;
-  self->window = NULL;
-
-  return CLUTTER_STAGE_WINDOW(self);
-}
-
 /*************************************************************************/
 static void
 clutter_stage_osx_init (ClutterStageOSX *self)
@@ -626,6 +616,35 @@ clutter_stage_osx_init (ClutterStageOSX *self)
   self->requisition_width  = 640;
   self->requisition_height = 480;
   self->acceptFocus = TRUE;
+
+  self->isHiding = false;
+  self->haveRealized = false;
+  self->view = NULL;
+  self->window = NULL;
+}
+
+static void
+clutter_stage_osx_set_property (GObject      *gobject,
+                                guint         prop_id,
+                                const GValue *value,
+                                GParamSpec   *pspec)
+{
+  ClutterStageOSX *self = CLUTTER_STAGE_OSX (gobject);
+
+  switch (prop_id)
+    {
+    case PROP_BACKEND:
+      self->backend = g_value_get_object (value);
+      break;
+
+    case PROP_WRAPPER:
+      self->wrapper = g_value_get_object (value);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
+      break;
+    }
 }
 
 static void
@@ -645,6 +664,10 @@ clutter_stage_osx_class_init (ClutterStageOSXClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  gobject_class->set_property = clutter_stage_osx_set_property;
   gobject_class->finalize = clutter_stage_osx_finalize;
   gobject_class->dispose = clutter_stage_osx_dispose;
+
+  g_object_class_override_property (gobject_class, PROP_BACKEND, "backend");
+  g_object_class_override_property (gobject_class, PROP_WRAPPER, "wrapper");
 }
