@@ -52,13 +52,6 @@ static ClutterBackendWayland *backend_singleton = NULL;
 
 G_DEFINE_TYPE (ClutterBackendWayland, _clutter_backend_wayland, CLUTTER_TYPE_BACKEND);
 
-static void
-clutter_backend_at_exit (void)
-{
-  if (backend_singleton)
-    g_object_run_dispose (G_OBJECT (backend_singleton));
-}
-
 static gboolean
 clutter_backend_wayland_pre_parse (ClutterBackend  *backend,
 				   GError         **error)
@@ -281,8 +274,6 @@ clutter_backend_wayland_post_parse (ClutterBackend  *backend,
   ClutterBackendWayland *backend_wayland = CLUTTER_BACKEND_WAYLAND (backend);
   EGLBoolean status;
 
-  g_atexit (clutter_backend_at_exit);
-
   /* TODO: expose environment variable/commandline option for this... */
   backend_wayland->wayland_display = wl_display_connect (NULL);
   if (!backend_wayland->wayland_display)
@@ -401,7 +392,7 @@ try_create_context (ClutterBackend  *backend,
           goto fail;
         }
 
-      CLUTTER_NOTE (GL, "Created EGL Context");
+      CLUTTER_NOTE (BACKEND, "Created EGL Context");
     }
 
   if (!eglMakeCurrent (backend_wayland->edpy,
@@ -493,12 +484,6 @@ clutter_backend_wayland_create_context (ClutterBackend  *backend,
   return status;
 }
 
-static void
-clutter_backend_wayland_ensure_context (ClutterBackend *backend,
-					ClutterStage   *stage)
-{
-}
-
 static void
 clutter_backend_wayland_redraw (ClutterBackend *backend,
 				ClutterStage   *stage)
@@ -514,11 +499,6 @@ clutter_backend_wayland_redraw (ClutterBackend *backend,
   _clutter_stage_wayland_redraw (CLUTTER_STAGE_WAYLAND (impl), stage);
 }
 
-static void
-clutter_backend_wayland_init_events (ClutterBackend *backend)
-{
-}
-
 static void
 clutter_backend_wayland_finalize (GObject *gobject)
 {
@@ -612,24 +592,6 @@ clutter_backend_wayland_get_features (ClutterBackend *backend)
   return flags;
 }
 
-static ClutterStageWindow *
-clutter_backend_wayland_create_stage (ClutterBackend  *backend,
-				      ClutterStage    *wrapper,
-				      GError         **error)
-{
-  ClutterBackendWayland *backend_wayland = CLUTTER_BACKEND_WAYLAND (backend);
-  ClutterStageWindow *stage;
-  ClutterStageWayland *stage_wayland;
-
-  stage = g_object_new (CLUTTER_TYPE_STAGE_WAYLAND, NULL);
-
-  stage_wayland = CLUTTER_STAGE_WAYLAND (stage);
-  stage_wayland->backend = backend_wayland;
-  stage_wayland->wrapper = wrapper;
-
-  return stage;
-}
-
 static void
 _clutter_backend_wayland_class_init (ClutterBackendWaylandClass *klass)
 {
@@ -640,13 +602,12 @@ _clutter_backend_wayland_class_init (ClutterBackendWaylandClass *klass)
   gobject_class->dispose     = clutter_backend_wayland_dispose;
   gobject_class->finalize    = clutter_backend_wayland_finalize;
 
+  backend_class->stage_window_type = CLUTTER_TYPE_STAGE_WAYLAND;
+
   backend_class->pre_parse        = clutter_backend_wayland_pre_parse;
   backend_class->post_parse       = clutter_backend_wayland_post_parse;
   backend_class->get_features     = clutter_backend_wayland_get_features;
-  backend_class->init_events      = clutter_backend_wayland_init_events;
-  backend_class->create_stage     = clutter_backend_wayland_create_stage;
   backend_class->create_context   = clutter_backend_wayland_create_context;
-  backend_class->ensure_context   = clutter_backend_wayland_ensure_context;
   backend_class->redraw           = clutter_backend_wayland_redraw;
 }
 
@@ -659,12 +620,6 @@ _clutter_backend_wayland_init (ClutterBackendWayland *backend_wayland)
   backend_wayland->drm_fd = -1;
 }
 
-GType
-_clutter_backend_impl_get_type (void)
-{
-  return _clutter_backend_wayland_get_type ();
-}
-
 EGLDisplay
 clutter_wayland_get_egl_display (void)
 {
