@@ -59,6 +59,7 @@
 #endif
 
 #include <cogl/cogl.h>
+#include <cogl/cogl-xlib.h>
 
 #include "clutter-backend.h"
 #include "clutter-debug.h"
@@ -91,9 +92,6 @@ static const gchar *atom_names[] = {
 
 #define N_ATOM_NAMES G_N_ELEMENTS (atom_names)
 
-/* singleton object */
-static ClutterBackendX11 *backend_singleton = NULL;
-
 /* various flags corresponding to pre init setup calls */
 static gboolean _no_xevent_retrieval = FALSE;
 static gboolean clutter_enable_xinput = FALSE;
@@ -279,6 +277,8 @@ clutter_backend_x11_create_device_manager (ClutterBackendX11 *backend_x11)
         }
 
       backend = CLUTTER_BACKEND (backend_x11);
+      backend->device_manager = backend_x11->device_manager;
+
       translator = CLUTTER_EVENT_TRANSLATOR (backend_x11->device_manager);
       _clutter_backend_add_event_translator (backend, translator);
     }
@@ -341,11 +341,14 @@ _clutter_backend_x11_post_parse (ClutterBackend  *backend,
                                  GError         **error)
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  ClutterSettings *settings;
+  Atom atoms[N_ATOM_NAMES];
+  double dpi;
 
   if (_foreign_dpy)
     backend_x11->xdpy = _foreign_dpy;
-  /*
-   * Only open connection if not already set by prior call to
+
+  /* Only open connection if not already set by prior call to
    * clutter_x11_set_display()
    */
   if (backend_x11->xdpy == NULL)
@@ -378,78 +381,66 @@ _clutter_backend_x11_post_parse (ClutterBackend  *backend,
 
   g_assert (backend_x11->xdpy != NULL);
 
-    {
-      ClutterSettings *settings;
-      Atom atoms[N_ATOM_NAMES];
-      double dpi;
+  CLUTTER_NOTE (BACKEND, "Getting the X screen");
 
-      CLUTTER_NOTE (BACKEND, "Getting the X screen");
+  settings = clutter_settings_get_default ();
 
-      settings = clutter_settings_get_default ();
+  /* Cogl needs to know the Xlib display connection for
+     CoglTexturePixmapX11 */
+  cogl_xlib_set_display (backend_x11->xdpy);
 
-      /* Cogl needs to know the Xlib display connection for
-         CoglTexturePixmapX11 */
-      cogl_xlib_set_display (backend_x11->xdpy);
+  /* add event filter for Cogl events */
+  clutter_x11_add_filter (cogl_xlib_filter, NULL);
 
-      /* add event filter for Cogl events */
-      clutter_x11_add_filter (cogl_xlib_filter, NULL);
+  if (clutter_screen == -1)
+    backend_x11->xscreen = DefaultScreenOfDisplay (backend_x11->xdpy);
+  else
+    backend_x11->xscreen = ScreenOfDisplay (backend_x11->xdpy,
+                                            clutter_screen);
 
-      if (clutter_screen == -1)
-        backend_x11->xscreen = DefaultScreenOfDisplay (backend_x11->xdpy);
-      else
-        backend_x11->xscreen = ScreenOfDisplay (backend_x11->xdpy,
-                                                clutter_screen);
+  backend_x11->xscreen_num = XScreenNumberOfScreen (backend_x11->xscreen);
+  backend_x11->xscreen_width = WidthOfScreen (backend_x11->xscreen);
+  backend_x11->xscreen_height = HeightOfScreen (backend_x11->xscreen);
 
-      backend_x11->xscreen_num = XScreenNumberOfScreen (backend_x11->xscreen);
-      backend_x11->xscreen_width = WidthOfScreen (backend_x11->xscreen);
-      backend_x11->xscreen_height = HeightOfScreen (backend_x11->xscreen);
+  backend_x11->xwin_root = RootWindow (backend_x11->xdpy,
+                                       backend_x11->xscreen_num);
 
-      backend_x11->xwin_root = RootWindow (backend_x11->xdpy,
-                                           backend_x11->xscreen_num);
+  backend_x11->display_name = g_strdup (clutter_display_name);
 
-      backend_x11->display_name = g_strdup (clutter_display_name);
+  dpi = (((double) DisplayHeight (backend_x11->xdpy, backend_x11->xscreen_num) * 25.4)
+      / (double) DisplayHeightMM (backend_x11->xdpy, backend_x11->xscreen_num));
 
-      dpi = (((double) DisplayHeight (backend_x11->xdpy, backend_x11->xscreen_num) * 25.4)
-            / (double) DisplayHeightMM (backend_x11->xdpy, backend_x11->xscreen_num));
+  g_object_set (settings, "font-dpi", (int) dpi * 1024, NULL);
 
-      g_object_set (settings, "font-dpi", (int) dpi * 1024, NULL);
+  /* create XSETTINGS client */
+  backend_x11->xsettings =
+    _clutter_xsettings_client_new (backend_x11->xdpy,
+                                   backend_x11->xscreen_num,
+                                   clutter_backend_x11_xsettings_notify,
+                                   NULL,
+                                   backend_x11);
 
-      /* create the device manager */
-      clutter_backend_x11_create_device_manager (backend_x11);
+  /* add event filter for XSETTINGS events */
+  clutter_x11_add_filter (xsettings_filter, backend_x11);
 
-      /* register keymap */
-      clutter_backend_x11_create_keymap (backend_x11);
+  if (clutter_synchronise)
+    XSynchronize (backend_x11->xdpy, True);
 
-      /* create XSETTINGS client */
-      backend_x11->xsettings =
-        _clutter_xsettings_client_new (backend_x11->xdpy,
-                                       backend_x11->xscreen_num,
-                                       clutter_backend_x11_xsettings_notify,
-                                       NULL,
-                                       backend_x11);
+  XInternAtoms (backend_x11->xdpy,
+                (char **) atom_names, N_ATOM_NAMES,
+                False, atoms);
 
-      /* add event filter for XSETTINGS events */
-      clutter_x11_add_filter (xsettings_filter, backend_x11);
-
-      if (clutter_synchronise)
-        XSynchronize (backend_x11->xdpy, True);
-
-      XInternAtoms (backend_x11->xdpy,
-                    (char **) atom_names, N_ATOM_NAMES,
-                    False, atoms);
-
-      backend_x11->atom_NET_WM_PID = atoms[0];
-      backend_x11->atom_NET_WM_PING = atoms[1];
-      backend_x11->atom_NET_WM_STATE = atoms[2];
-      backend_x11->atom_NET_WM_STATE_FULLSCREEN = atoms[3];
-      backend_x11->atom_NET_WM_USER_TIME = atoms[4];
-      backend_x11->atom_WM_PROTOCOLS = atoms[5];
-      backend_x11->atom_WM_DELETE_WINDOW = atoms[6];
-      backend_x11->atom_XEMBED = atoms[7];
-      backend_x11->atom_XEMBED_INFO = atoms[8];
-      backend_x11->atom_NET_WM_NAME = atoms[9];
-      backend_x11->atom_UTF8_STRING = atoms[10];
-    }
+  backend_x11->atom_NET_WM_PID = atoms[0];
+  backend_x11->atom_NET_WM_PING = atoms[1];
+  backend_x11->atom_NET_WM_STATE = atoms[2];
+  backend_x11->atom_NET_WM_STATE_FULLSCREEN = atoms[3];
+  backend_x11->atom_NET_WM_USER_TIME = atoms[4];
+  backend_x11->atom_WM_PROTOCOLS = atoms[5];
+  backend_x11->atom_WM_DELETE_WINDOW = atoms[6];
+  backend_x11->atom_XEMBED = atoms[7];
+  backend_x11->atom_XEMBED_INFO = atoms[8];
+  backend_x11->atom_NET_WM_NAME = atoms[9];
+  backend_x11->atom_UTF8_STRING = atoms[10];
 
   g_free (clutter_display_name);
 
@@ -464,14 +455,46 @@ _clutter_backend_x11_post_parse (ClutterBackend  *backend,
   return TRUE;
 }
 
-
-static void
-clutter_backend_x11_init_events (ClutterBackend *backend)
+void
+_clutter_backend_x11_events_init (ClutterBackend *backend)
 {
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+
   CLUTTER_NOTE (EVENT, "initialising the event loop");
 
+  /* the event source is optional */
   if (!_no_xevent_retrieval)
-    _clutter_backend_x11_events_init (backend);
+    {
+      GSource *source;
+
+      source = _clutter_x11_event_source_new (backend_x11);
+
+      /* default priority for events
+       *
+       * XXX - at some point we'll have a common EventSource API that
+       * is created by the backend, and this code will most likely go
+       * into the default implementation of ClutterBackend
+       */
+      g_source_set_priority (source, CLUTTER_PRIORITY_EVENTS);
+
+      /* attach the source to the default context, and transfer the
+       * ownership to the GMainContext itself
+       */
+      g_source_attach (source, NULL);
+      g_source_unref (source);
+
+      backend_x11->event_source = source;
+    }
+
+  /* create the device manager; we need this because we can effectively
+   * choose between core+XI1 and XI2 input events
+   */
+  clutter_backend_x11_create_device_manager (backend_x11);
+
+  /* register keymap; unless we create a generic Keymap object, I'm
+   * afraid this will have to stay
+   */
+  clutter_backend_x11_create_keymap (backend_x11);
 }
 
 static const GOptionEntry entries[] =
@@ -525,57 +548,24 @@ clutter_backend_x11_finalize (GObject *gobject)
 
   XCloseDisplay (backend_x11->xdpy);
 
-  if (backend_singleton)
-    backend_singleton = NULL;
-
   G_OBJECT_CLASS (clutter_backend_x11_parent_class)->finalize (gobject);
 }
 
 static void
 clutter_backend_x11_dispose (GObject *gobject)
 {
-  ClutterBackendX11   *backend_x11 = CLUTTER_BACKEND_X11 (gobject);
-  ClutterStageManager *stage_manager;
-
-  CLUTTER_NOTE (BACKEND, "Disposing the of stages");
-  stage_manager = clutter_stage_manager_get_default ();
-
-  g_object_unref (stage_manager);
-
-  CLUTTER_NOTE (BACKEND, "Removing the event source");
-  _clutter_backend_x11_events_uninit (CLUTTER_BACKEND (backend_x11));
-
   G_OBJECT_CLASS (clutter_backend_x11_parent_class)->dispose (gobject);
 }
 
-static GObject *
-clutter_backend_x11_constructor (GType                  gtype,
-                                 guint                  n_params,
-                                 GObjectConstructParam *params)
-{
-  GObjectClass *parent_class;
-  GObject *retval;
-
-  if (backend_singleton == NULL)
-    {
-      parent_class = G_OBJECT_CLASS (clutter_backend_x11_parent_class);
-      retval = parent_class->constructor (gtype, n_params, params);
-
-      backend_singleton = CLUTTER_BACKEND_X11 (retval);
-
-      return retval;
-    }
-
-  g_warning ("Attempting to create a new backend object. This should "
-             "never happen, so we return the singleton instance.");
-
-  return g_object_ref (backend_singleton);
-}
-
 static ClutterFeatureFlags
 clutter_backend_x11_get_features (ClutterBackend *backend)
 {
-  return CLUTTER_FEATURE_STAGE_USER_RESIZE | CLUTTER_FEATURE_STAGE_CURSOR;
+  ClutterFeatureFlags flags = CLUTTER_FEATURE_STAGE_USER_RESIZE
+                            | CLUTTER_FEATURE_STAGE_CURSOR;
+
+  flags |= CLUTTER_BACKEND_CLASS (clutter_backend_x11_parent_class)->get_features (backend);
+
+  return flags;
 }
 
 static void
@@ -601,16 +591,6 @@ clutter_backend_x11_free_event_data (ClutterBackend *backend,
     _clutter_event_x11_free (event_x11);
 }
 
-static ClutterDeviceManager *
-clutter_backend_x11_get_device_manager (ClutterBackend *backend)
-{
-  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
-
-  clutter_backend_x11_create_device_manager (backend_x11);
-
-  return backend_x11->device_manager;
-}
-
 static void
 update_last_event_time (ClutterBackendX11 *backend_x11,
                         XEvent            *xevent)
@@ -705,25 +685,135 @@ clutter_backend_x11_translate_event (ClutterBackend *backend,
   return parent_class->translate_event (backend, native, event);
 }
 
+static CoglRenderer *
+clutter_backend_x11_get_renderer (ClutterBackend  *backend,
+                                  GError         **error)
+{
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  Display *xdisplay = backend_x11->xdpy;
+  CoglRenderer *renderer;
+
+  CLUTTER_NOTE (BACKEND, "Creating a new Xlib renderer");
+
+  renderer = cogl_renderer_new ();
+
+  /* set the display object we're using */
+  cogl_xlib_renderer_set_foreign_display (renderer, xdisplay);
+
+  return renderer;
+}
+
+static CoglDisplay *
+clutter_backend_x11_get_display (ClutterBackend  *backend,
+                                 CoglRenderer    *renderer,
+                                 CoglSwapChain   *swap_chain,
+                                 GError         **error)
+{
+  CoglOnscreenTemplate *onscreen_template;
+  GError *internal_error = NULL;
+  CoglDisplay *display;
+  gboolean res;
+
+  CLUTTER_NOTE (BACKEND, "Alpha on Cogl swap chain: %s",
+                clutter_enable_argb ? "enabled" : "disabled");
+
+  cogl_swap_chain_set_has_alpha (swap_chain, clutter_enable_argb);
+
+  onscreen_template = cogl_onscreen_template_new (swap_chain);
+
+  res = cogl_renderer_check_onscreen_template (renderer,
+                                               onscreen_template,
+                                               &internal_error);
+  if (!res && clutter_enable_argb)
+    {
+      CLUTTER_NOTE (BACKEND,
+                    "Creation of a context with a ARGB visual failed: %s",
+                    internal_error != NULL ? internal_error->message
+                                           : "Unknown reason");
+
+      g_clear_error (&internal_error);
+
+      /* It's possible that the current renderer doesn't support transparency
+       * in a swap_chain so lets see if we can fallback to not having any
+       * transparency...
+       *
+       * XXX: It might be nice to have a CoglRenderer feature we could
+       * explicitly check for ahead of time.
+       */
+      clutter_enable_argb = FALSE;
+      cogl_swap_chain_set_has_alpha (swap_chain, FALSE);
+      res = cogl_renderer_check_onscreen_template (renderer,
+                                                   onscreen_template,
+                                                   &internal_error);
+    }
+
+  if (!res)
+    {
+      g_set_error_literal (error, CLUTTER_INIT_ERROR,
+                           CLUTTER_INIT_ERROR_BACKEND,
+                           internal_error->message);
+
+      g_error_free (internal_error);
+      cogl_object_unref (onscreen_template);
+
+      return NULL;
+    }
+
+  display = cogl_display_new (renderer, onscreen_template);
+  cogl_object_unref (onscreen_template);
+
+  return display;
+}
+
+static ClutterStageWindow *
+clutter_backend_x11_create_stage (ClutterBackend  *backend,
+				  ClutterStage    *wrapper,
+				  GError         **error)
+{
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  ClutterEventTranslator *translator;
+  ClutterStageWindow *stage;
+
+  stage = g_object_new (CLUTTER_TYPE_STAGE_X11,
+			"backend", backend,
+			"wrapper", wrapper,
+			NULL);
+
+  /* the X11 stage does event translation */
+  translator = CLUTTER_EVENT_TRANSLATOR (stage);
+  _clutter_backend_add_event_translator (backend, translator);
+
+  CLUTTER_NOTE (MISC, "X11 stage created (display:%p, screen:%d, root:%u)",
+                backend_x11->xdpy,
+                backend_x11->xscreen_num,
+                (unsigned int) backend_x11->xwin_root);
+
+  return stage;
+}
+
 static void
 clutter_backend_x11_class_init (ClutterBackendX11Class *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterBackendClass *backend_class = CLUTTER_BACKEND_CLASS (klass);
 
-  gobject_class->constructor = clutter_backend_x11_constructor;
   gobject_class->dispose = clutter_backend_x11_dispose;
   gobject_class->finalize = clutter_backend_x11_finalize;
 
+  backend_class->stage_window_type = CLUTTER_TYPE_STAGE_X11;
+
   backend_class->pre_parse = _clutter_backend_x11_pre_parse;
   backend_class->post_parse = _clutter_backend_x11_post_parse;
-  backend_class->init_events = clutter_backend_x11_init_events;
   backend_class->add_options = clutter_backend_x11_add_options;
   backend_class->get_features = clutter_backend_x11_get_features;
-  backend_class->get_device_manager = clutter_backend_x11_get_device_manager;
+
   backend_class->copy_event_data = clutter_backend_x11_copy_event_data;
   backend_class->free_event_data = clutter_backend_x11_free_event_data;
   backend_class->translate_event = clutter_backend_x11_translate_event;
+
+  backend_class->get_renderer = clutter_backend_x11_get_renderer;
+  backend_class->get_display = clutter_backend_x11_get_display;
+  backend_class->create_stage = clutter_backend_x11_create_stage;
 }
 
 static void
@@ -783,13 +873,21 @@ clutter_x11_untrap_x_errors (void)
 Display *
 clutter_x11_get_default_display (void)
 {
-  if (!backend_singleton)
+  ClutterBackend *backend = clutter_get_default_backend ();
+
+  if (backend == NULL)
     {
-      g_critical ("X11 backend has not been initialised");
+      g_critical ("The Clutter backend has not been initialised");
       return NULL;
     }
 
-  return backend_singleton->xdpy;
+  if (!CLUTTER_IS_BACKEND_X11 (backend))
+    {
+      g_critical ("The Clutter backend is not a X11 backend");
+      return NULL;
+    }
+
+  return CLUTTER_BACKEND_X11 (backend)->xdpy;
 }
 
 /**
@@ -912,13 +1010,21 @@ clutter_x11_has_event_retrieval (void)
 int
 clutter_x11_get_default_screen (void)
 {
-  if (!backend_singleton)
+ ClutterBackend *backend = clutter_get_default_backend ();
+
+  if (backend == NULL)
     {
-      g_critical ("X11 backend has not been initialised");
+      g_critical ("The Clutter backend has not been initialised");
       return 0;
     }
 
-  return backend_singleton->xscreen_num;
+  if (!CLUTTER_IS_BACKEND_X11 (backend))
+    {
+      g_critical ("The Clutter backend is not a X11 backend");
+      return 0;
+    }
+
+  return CLUTTER_BACKEND_X11 (backend)->xscreen_num;
 }
 
 /**
@@ -933,13 +1039,21 @@ clutter_x11_get_default_screen (void)
 Window
 clutter_x11_get_root_window (void)
 {
-  if (!backend_singleton)
+ ClutterBackend *backend = clutter_get_default_backend ();
+
+  if (backend == NULL)
     {
-      g_critical ("X11 backend has not been initialised");
+      g_critical ("The Clutter backend has not been initialised");
       return None;
     }
 
-  return backend_singleton->xwin_root;
+  if (!CLUTTER_IS_BACKEND_X11 (backend))
+    {
+      g_critical ("The Clutter backend is not a X11 backend");
+      return None;
+    }
+
+  return CLUTTER_BACKEND_X11 (backend)->xwin_root;
 }
 
 /**
@@ -956,21 +1070,31 @@ clutter_x11_add_filter (ClutterX11FilterFunc func,
                         gpointer             data)
 {
   ClutterX11EventFilter *filter;
+  ClutterBackend *backend = clutter_get_default_backend ();
+  ClutterBackendX11 *backend_x11;
 
   g_return_if_fail (func != NULL);
 
-  if (!backend_singleton)
+  if (backend == NULL)
     {
-      g_critical ("X11 backend has not been initialised");
+      g_critical ("The Clutter backend has not been initialised");
       return;
     }
 
+  if (!CLUTTER_IS_BACKEND_X11 (backend))
+    {
+      g_critical ("The Clutter backend is not a X11 backend");
+      return;
+    }
+
+  backend_x11 = CLUTTER_BACKEND_X11 (backend);
+
   filter = g_new0 (ClutterX11EventFilter, 1);
   filter->func = func;
   filter->data = data;
 
-  backend_singleton->event_filters =
-    g_slist_append (backend_singleton->event_filters, filter);
+  backend_x11->event_filters =
+    g_slist_append (backend_x11->event_filters, filter);
 
   return;
 }
@@ -990,10 +1114,26 @@ clutter_x11_remove_filter (ClutterX11FilterFunc func,
 {
   GSList                *tmp_list, *this;
   ClutterX11EventFilter *filter;
+  ClutterBackend *backend = clutter_get_default_backend ();
+  ClutterBackendX11 *backend_x11;
 
   g_return_if_fail (func != NULL);
 
-  tmp_list = backend_singleton->event_filters;
+  if (backend == NULL)
+    {
+      g_critical ("The Clutter backend has not been initialised");
+      return;
+    }
+
+  if (!CLUTTER_IS_BACKEND_X11 (backend))
+    {
+      g_critical ("The Clutter backend is not a X11 backend");
+      return;
+    }
+
+  backend_x11 = CLUTTER_BACKEND_X11 (backend);
+
+  tmp_list = backend_x11->event_filters;
 
   while (tmp_list)
     {
@@ -1003,8 +1143,8 @@ clutter_x11_remove_filter (ClutterX11FilterFunc func,
 
       if (filter->func == func && filter->data == data)
         {
-          backend_singleton->event_filters =
-            g_slist_remove_link (backend_singleton->event_filters, this);
+          backend_x11->event_filters =
+            g_slist_remove_link (backend_x11->event_filters, this);
 
           g_slist_free_1 (this);
           g_free (filter);
@@ -1051,10 +1191,21 @@ gboolean
 clutter_x11_has_xinput (void)
 {
 #if defined(HAVE_XINPUT) || defined(HAVE_XINPUT_2)
-  if (backend_singleton != NULL)
-    return backend_singleton->has_xinput;
+ ClutterBackend *backend = clutter_get_default_backend ();
 
-  return FALSE;
+  if (backend == NULL)
+    {
+      g_critical ("The Clutter backend has not been initialised");
+      return FALSE;
+    }
+
+  if (!CLUTTER_IS_BACKEND_X11 (backend))
+    {
+      g_critical ("The Clutter backend is not a X11 backend.");
+      return FALSE;
+    }
+
+  return CLUTTER_BACKEND_X11 (backend)->has_xinput;
 #else
   return FALSE;
 #endif
@@ -1086,6 +1237,8 @@ clutter_x11_has_composite_extension (void)
     }
 
   dpy = clutter_x11_get_default_display();
+  if (dpy == NULL)
+    return FALSE;
 
   if (XCompositeQueryExtension (dpy, &event, &error))
     {
@@ -1158,15 +1311,7 @@ clutter_x11_get_use_argb_visual (void)
 XVisualInfo *
 _clutter_backend_x11_get_visual_info (ClutterBackendX11 *backend_x11)
 {
-  ClutterBackendX11Class *klass;
-
-  g_return_val_if_fail (CLUTTER_IS_BACKEND_X11 (backend_x11), NULL);
-
-  klass = CLUTTER_BACKEND_X11_GET_CLASS (backend_x11);
-  if (klass->get_visual_info)
-    return klass->get_visual_info (backend_x11);
-
-  return NULL;
+  return cogl_clutter_winsys_xlib_get_visual_info ();
 }
 
 /**
@@ -1185,8 +1330,16 @@ XVisualInfo *
 clutter_x11_get_visual_info (void)
 {
   ClutterBackendX11 *backend_x11;
+  ClutterBackend *backend;
 
-  backend_x11 = CLUTTER_BACKEND_X11 (clutter_get_default_backend ());
+  backend = clutter_get_default_backend ();
+  if (!CLUTTER_IS_BACKEND_X11 (backend))
+    {
+      g_critical ("The Clutter backend is not a X11 backend.");
+      return NULL;
+    }
+
+  backend_x11 = CLUTTER_BACKEND_X11 (backend);
 
   return _clutter_backend_x11_get_visual_info (backend_x11);
 }
