@@ -8,6 +8,8 @@
 
 static const ClutterColor stage_color = { 0x0, 0x0, 0x0, 0xff };
 
+static TestConformGLFunctions gl_functions;
+
 #define QUAD_WIDTH 20
 
 #define RED 0
@@ -39,7 +41,6 @@ check_pixel (TestState *state, int x, int y, guint32 color)
 
   /* See what we got... */
 
-  /* NB: glReadPixels is done in GL screen space so y = 0 is at the bottom */
   y_off = y * QUAD_WIDTH + (QUAD_WIDTH / 2);
   x_off = x * QUAD_WIDTH + (QUAD_WIDTH / 2);
 
@@ -70,14 +71,22 @@ test_material_with_primitives (TestState *state,
                                int x, int y,
                                guint32 color)
 {
-  CoglTextureVertex verts[4] = {
-    { .x = 0,          .y = 0,          .z = 0 },
-    { .x = 0,          .y = QUAD_WIDTH, .z = 0 },
-    { .x = QUAD_WIDTH, .y = QUAD_WIDTH, .z = 0 },
-    { .x = QUAD_WIDTH, .y = 0,          .z = 0 },
-  };
+  CoglTextureVertex verts[4];
   CoglHandle vbo;
 
+  verts[0].x = 0;
+  verts[0].y = 0;
+  verts[0].z = 0;
+  verts[1].x = 0;
+  verts[1].y = QUAD_WIDTH;
+  verts[1].z = 0;
+  verts[2].x = QUAD_WIDTH;
+  verts[2].y = QUAD_WIDTH;
+  verts[2].z = 0;
+  verts[3].x = QUAD_WIDTH;
+  verts[3].y = 0;
+  verts[3].z = 0;
+
   cogl_push_matrix ();
 
   cogl_translate (x * QUAD_WIDTH, y * QUAD_WIDTH, 0);
@@ -127,6 +136,17 @@ test_invalid_texture_layers (TestState *state, int x, int y)
   test_material_with_primitives (state, x, y, 0xffffffff);
 }
 
+#ifdef COGL_HAS_GLES2
+static gboolean
+using_gles2_driver (void)
+{
+  /* FIXME: This should probably be replaced with some way to query
+     the driver from Cogl */
+  return g_str_has_prefix ((const char *) gl_functions.glGetString (GL_VERSION),
+                           "OpenGL ES 2");
+}
+#endif
+
 static void
 test_using_all_layers (TestState *state, int x, int y)
 {
@@ -154,23 +174,29 @@ test_using_all_layers (TestState *state, int x, int y)
 
   /* FIXME: Cogl doesn't provide a way to query the maximum number of
      texture layers so for now we'll just ask GL directly. */
-#ifdef HAVE_COGL_GLES2
-  {
-    GLint n_image_units, n_attribs;
-    /* GLES 2 doesn't have GL_MAX_TEXTURE_UNITS and it uses
-       GL_MAX_TEXTURE_IMAGE_UNITS instead */
-    glGetIntegerv (GL_MAX_TEXTURE_IMAGE_UNITS, &n_image_units);
-    /* Cogl needs a vertex attrib for each layer to upload the texture
-       coordinates */
-    glGetIntegerv (GL_MAX_VERTEX_ATTRIBS, &n_attribs);
-    /* We can't use two of the attribs because they are used by the
-       position and color */
-    n_attribs -= 2;
-    n_layers = MIN (n_attribs, n_image_units);
-  }
-#else
-  glGetIntegerv (GL_MAX_TEXTURE_UNITS, &n_layers);
+#ifdef COGL_HAS_GLES2
+  if (using_gles2_driver ())
+    {
+      GLint n_image_units, n_attribs;
+      /* GLES 2 doesn't have GL_MAX_TEXTURE_UNITS and it uses
+         GL_MAX_TEXTURE_IMAGE_UNITS instead */
+      gl_functions.glGetIntegerv (GL_MAX_TEXTURE_IMAGE_UNITS, &n_image_units);
+      /* Cogl needs a vertex attrib for each layer to upload the texture
+         coordinates */
+      gl_functions.glGetIntegerv (GL_MAX_VERTEX_ATTRIBS, &n_attribs);
+      /* We can't use two of the attribs because they are used by the
+         position and color */
+      n_attribs -= 2;
+      n_layers = MIN (n_attribs, n_image_units);
+    }
+  else
 #endif
+    {
+#if defined(COGL_HAS_GLES1) || defined(COGL_HAS_GL)
+      gl_functions.glGetIntegerv (GL_MAX_TEXTURE_UNITS, &n_layers);
+#endif
+    }
+
   /* FIXME: is this still true? */
   /* Cogl currently can't cope with more than 32 layers so we'll also
      limit the maximum to that. */
@@ -222,6 +248,51 @@ test_invalid_texture_layers_with_constant_colors (TestState *state,
   test_material_with_primitives (state, x, y, 0x0000ffff);
 }
 
+static void
+basic_ref_counting_destroy_cb (void *user_data)
+{
+  gboolean *destroyed_flag = user_data;
+
+  g_assert (*destroyed_flag == FALSE);
+
+  *destroyed_flag = TRUE;
+}
+
+static void
+test_basic_ref_counting (void)
+{
+  CoglMaterial *material_parent;
+  gboolean parent_destroyed = FALSE;
+  CoglMaterial *material_child;
+  gboolean child_destroyed = FALSE;
+  static CoglUserDataKey user_data_key;
+
+  /* This creates a material with a copy and then just unrefs them
+     both without setting them as a source. They should immediately be
+     freed. We can test whether they were freed or not by registering
+     a destroy callback with some user data */
+
+  material_parent = cogl_material_new ();
+  /* Set some user data so we can detect when the material is
+     destroyed */
+  cogl_object_set_user_data (COGL_OBJECT (material_parent),
+                             &user_data_key,
+                             &parent_destroyed,
+                             basic_ref_counting_destroy_cb);
+
+  material_child = cogl_material_copy (material_parent);
+  cogl_object_set_user_data (COGL_OBJECT (material_child),
+                             &user_data_key,
+                             &child_destroyed,
+                             basic_ref_counting_destroy_cb);
+
+  cogl_object_unref (material_child);
+  cogl_object_unref (material_parent);
+
+  g_assert (parent_destroyed);
+  g_assert (child_destroyed);
+}
+
 static void
 on_paint (ClutterActor *actor, TestState *state)
 {
@@ -235,6 +306,8 @@ on_paint (ClutterActor *actor, TestState *state)
                          2, 0 /* position */
                          );
 
+  test_basic_ref_counting ();
+
   /* Comment this out if you want visual feedback for what this test paints */
 #if 1
   clutter_main_quit ();
@@ -258,7 +331,9 @@ test_cogl_materials (TestConformSimpleFixture *fixture,
   ClutterActor *group;
   guint idle_source;
 
-  stage = clutter_stage_get_default ();
+  test_conform_get_gl_functions (&gl_functions);
+
+  stage = clutter_stage_new ();
 
   clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);
   clutter_actor_get_geometry (stage, &state.stage_geom);
@@ -279,7 +354,8 @@ test_cogl_materials (TestConformSimpleFixture *fixture,
 
   g_source_remove (idle_source);
 
+  clutter_actor_destroy (stage);
+
   if (g_test_verbose ())
     g_print ("OK\n");
 }
-
