@@ -167,7 +167,7 @@ test_cogl_vertex_buffer_contiguous (TestConformSimpleFixture *fixture,
     0x00, 0xff, 0x00, 0xff
   };
 
-  stage = clutter_stage_get_default ();
+  stage = clutter_stage_new ();
 
   clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_clr);
   clutter_actor_get_geometry (stage, &state.stage_geom);
@@ -251,7 +251,8 @@ test_cogl_vertex_buffer_contiguous (TestConformSimpleFixture *fixture,
 
   g_source_remove (idle_source);
 
+  clutter_actor_destroy (stage);
+
   if (g_test_verbose ())
     g_print ("OK\n");
 }
-
