@@ -319,12 +319,15 @@ test_clip_main (int argc, char **argv)
   ClutterActor *stub_actor, *label;
   gchar *file;
 
-  clutter_init (&argc, &argv);
+  if (clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS)
+    return 1;
 
   data.current_clip.type = CLIP_NONE;
   data.clips = NULL;
 
-  data.stage = clutter_stage_get_default ();
+  data.stage = clutter_stage_new ();
+  clutter_stage_set_title (CLUTTER_STAGE (data.stage), "Clipping");
+  g_signal_connect (data.stage, "destroy", G_CALLBACK (clutter_main_quit), NULL);
 
   stub_actor = clutter_rectangle_new ();
   clutter_container_add (CLUTTER_CONTAINER (data.stage), stub_actor, NULL);
@@ -365,3 +368,9 @@ test_clip_main (int argc, char **argv)
 
   return 0;
 }
+
+G_MODULE_EXPORT const char *
+test_clip_describe (void)
+{
+  return "Actor clipping with various techniques";
+}
