@@ -1,5 +1,3 @@
-#undef CLUTTER_DISABLE_DEPRECATED
-
 #include <clutter/clutter.h>
 
 #include <errno.h>
@@ -57,9 +55,11 @@ test_fbo_main (int argc, char *argv[])
   if (clutter_feature_available (CLUTTER_FEATURE_OFFSCREEN) == FALSE)
     g_error("This test requires CLUTTER_FEATURE_OFFSCREEN");
 
-  stage = clutter_stage_get_default ();
+  stage = clutter_stage_new ();
   clutter_actor_set_size (stage, STAGE_WIDTH, STAGE_HEIGHT);
   clutter_stage_set_color (CLUTTER_STAGE (stage), &blue);
+  clutter_stage_set_title (CLUTTER_STAGE (stage), "Texture from Actor");
+  g_signal_connect (stage, "destroy", G_CALLBACK (clutter_main_quit), NULL);
 
   /* Create the first source */
   onscreen_source = make_source();
@@ -99,3 +99,9 @@ test_fbo_main (int argc, char *argv[])
 
   return 0;
 }
+
+G_MODULE_EXPORT const char *
+test_fbo_describe (void)
+{
+  return "Create a texture from an actor.";
+}
